require 'spec_helper'
require 'db_obfuscation/obfuscator'

module DbObfuscation
  describe Obfuscator do
    context 'medicaid recipent identification number' do
      it 'uses faker numerify' do
        expect(FFaker).to receive(:numerify).with('############')
        described_class.obfuscate(:medicaid_id_strategy)
      end

      it 'generates random 12 digits number' do
        m_id = described_class.obfuscate(:medicaid_id_strategy)
        expect(m_id.chars.size).to eq(12)
      end
    end

    it 'obfuscates school strategy with a fake school name' do
      expect(FFaker::Education).to receive(:school)
      described_class.obfuscate(:school_strategy)
    end

    context 'unique strategy' do
      it 'provides unique strings' do
        expect(FFaker::Name).to receive(:first_name)#.and_return 'whatever'
        expect(FFaker).to receive(:letterify).with('??????????')
        expect(FFaker::Name).to receive(:last_name)
        described_class.obfuscate(:unique_name_strategy)
      end

      it 'provides names comprised of first, random middle and last names' do
        allow(FFaker::Name).to receive(:first_name).and_return('First')
        allow(FFaker).to receive(:letterify).with('??????????').and_return('abc')
        allow(FFaker::Name).to receive(:last_name).and_return('Last')
        name = 'First abc Last'

        expect(described_class.obfuscate(:unique_name_strategy)).to eq(name)
      end
    end

    it 'obfuscates gender as unknown' do
      expect(described_class.obfuscate(:gender_strategy)).to eq 'Unknown'
    end

    it 'obfuscates default strategy as a random word' do
      expect(FFaker::Lorem).to receive(:word).and_return('fake_word')
      obfuscated_fake_word = described_class.obfuscate(
        :default_strategy)
      expect(obfuscated_fake_word).to eq 'fake_word'
    end

    it 'obfuscates name strategy as fake name' do
      expect(FFaker::Name).to receive(:name).and_return('fake_name')
      obfuscated_fake_name = described_class.obfuscate(
        :name_strategy)
      expect(obfuscated_fake_name).to eq 'fake_name'
    end

    it 'obfuscates first_name strategy as fake first name' do
      expect(FFaker::Name).to receive(:first_name).and_return('fake_first_name')
      obfuscated_fake_first_name = described_class.obfuscate(
        :first_name_strategy)
      expect(obfuscated_fake_first_name).to eq 'fake_first_name'
    end

    it 'obfuscates last_name strategy as fake last name' do
      expect(FFaker::Name).to receive(:last_name).and_return('fake_last_name')
      obfuscated_fake_last_name = described_class.obfuscate(
        :last_name_strategy)
      expect(obfuscated_fake_last_name).to eq 'fake_last_name'
    end

    it 'obfuscates address strategy as fake address with city/state/zip' do
      expect(FFaker::Address).to receive(:street_address).
        and_return('fake_street_address')
      expect(FFaker::AddressUS).to receive(:zip_code).and_return('10000')
      expect(FFaker::Address).to receive(:city).and_return('New York')
      expect(FFaker::AddressUS).to receive(:state_abbr).and_return('NY')
      obfuscated_fake_address = described_class.obfuscate(
        :address_strategy)
      expect(obfuscated_fake_address).
        to eq 'fake_street_address New York NY 10000'
    end

    it 'obfuscates ssn strategy as fake ssn' do
      expect(FFaker::SSN).to receive(:ssn).and_return('123-456-7890')
      obfuscated_fake_ssn = described_class.obfuscate(
        :ssn_strategy)
      expect(obfuscated_fake_ssn).to eq '123-456-7890'
    end

    it 'obfuscates email strategy as fake email' do
      expect(FFaker::Internet).to receive(:safe_email).
        and_return('fake_email@fake_email.com')
      obfuscated_fake_email = described_class.obfuscate(
        :email_strategy)
      expect(obfuscated_fake_email).to eq 'fake_email@fake_email.com'
    end

    it 'obfuscsates phone number strategy as fake phone number' do
      expect(FFaker::PhoneNumber).to receive(:phone_number).
        and_return('123-456-7890')
      obfuscated_fake_phone_number = described_class.obfuscate(
        :phone_number_strategy)
      expect(obfuscated_fake_phone_number).to eq '123-456-7890'
    end

    it 'obfuscates nil strategy as nil value' do
      obfuscated_string = described_class.obfuscate(
        :nil_strategy)
      expect(obfuscated_string).to eq nil
    end

    it 'obfuscates sentence strategy as fake sentence' do
      expect(FFaker::Lorem).to receive(:sentence).
        and_return('this is a fake sentence')
      obfuscated_string = described_class.obfuscate(
        :sentence_strategy)
      expect(obfuscated_string).to eq 'this is a fake sentence'
    end

    it 'obfuscates paragraph strategy as fake paragraph' do
      expect(FFaker::Lorem).to receive(:paragraph).
        and_return('this is a fake paragraph')
      obfuscated_string = described_class.obfuscate(
        :paragraph_strategy)
      expect(obfuscated_string).to eq 'this is a fake paragraph'
    end

    it 'obfuscates driving license strategy as fake driving license' do
      expect(FFaker::Identification).to receive(:drivers_license).
        and_return('123-456-7890')
      obfuscated_string = described_class.obfuscate(
        :driving_license_strategy)
      expect(obfuscated_string).to eq '123-456-7890'
    end

    it 'the date strategy returns random number of days' do
      expect(described_class.obfuscate(:date_strategy)).to be_between(31,240)
    end

    it 'obfuscates suffix strategy as fake suffix' do
      expect(FFaker::Name).to receive(:suffix).
        and_return('Jr')
      obfuscated_string = described_class.obfuscate(
        :suffix_strategy)
      expect(obfuscated_string).to eq 'Jr'
    end
  end
end
