require 'spec_helper'
require 'db_obfuscation/query_builder'

module DbObfuscation
  describe QueryBuilder do
    describe '.multi_update_sql' do
      let(:sql_query) do
        <<-SQL.gsub(/\s{2,}/,' ')
        UPDATE "table_1"
        SET "field_1" = CASE
          WHEN "id" = '1' AND "field_1" IS NOT NULL
            AND "field_1" != '' THEN 'field_1_1'
          WHEN "id" = '2' AND "field_1" IS NOT NULL
            AND "field_1" != '' THEN 'field_1_2'
          WHEN "id" = '3' AND "field_1" IS NOT NULL
            AND "field_1" != '' THEN 'field_1_3'
          ELSE
            "field_1"
        END,
        "field_2" = CASE
          WHEN "id" = '1' AND "field_2" IS NOT NULL
            AND "field_2" != '' THEN 'field_2_1'
          WHEN "id" = '2' AND "field_2" IS NOT NULL
            AND "field_2" != '' THEN 'field_2_2'
          WHEN "id" = '3' AND "field_2" IS NOT NULL
            AND "field_2" != '' THEN 'field_2_3'
          ELSE
            "field_2"
        END
        WHERE "id" IN ('1','2','3')
        SQL
      end

      it 'build sql query for multiple row updates' do
        columns_and_values = [
          { field_1: "'field_1_1'", field_2: "'field_2_1'", id: 1 },
          { field_1: "'field_1_2'", field_2: "'field_2_2'", id: 2 },
          { field_1: "'field_1_3'", field_2: "'field_2_3'", id: 3 },
        ]

        expect(described_class.multi_update_sql(
          :table_1, columns_and_values)).to eq sql_query.strip
      end

      it 'handles single row update' do
        sql_query = <<-SQL.gsub(/\s{2,}/,' ')
          UPDATE "table_1"
          SET "field_1" = CASE
            WHEN "id" = '1' AND "field_1" IS NOT NULL
              AND "field_1" != '' THEN 'field_1_1'
            ELSE
              "field_1"
          END,
          "field_2" = CASE
            WHEN "id" = '1' AND "field_2" IS NOT NULL
              AND "field_2" != '' THEN 'field_2_1'
            ELSE
              "field_2"
          END
          WHERE "id" IN ('1')
        SQL
        columns_and_values = [
          { field_1: "'field_1_1'", field_2: "'field_2_1'", id: 1 },
        ]
        expect(described_class.multi_update_sql(
          :table_1,columns_and_values)).to eq sql_query.strip
      end
    end

    describe '.where_sql' do
      it 'generates where sql query' do
        sql_query = "WHERE \"id\" IN ('1','2')"
        columns_and_values = [
          { first_name: 'first1', last_name: 'last1', id: 1 },
          { first_name: 'first2', last_name: 'last2', id: 2 }]
        expect(described_class.where_sql(columns_and_values)).to eq sql_query
      end

      it 'generates sql query based on the user supplied constraints' do
        sql_query = "WHERE \"name\" IN ('1','2')"
        columns_and_values = [
          { first_name: "'first1'", last_name: "'last1'", name: 1 },
          { first_name: "'first2'", last_name: "'last2'", name: 2 }]
        expect(described_class.where_sql(columns_and_values)).to eq sql_query
      end
    end

    describe '.case_sql' do
      let(:sql) do
        <<-SQL.gsub(/\s{2,}/,' ')
        SET "first_name" = CASE
          WHEN "id" = '1' AND "first_name" IS NOT NULL
            AND "first_name" != '' THEN 'first1'
          WHEN "id" = '2' AND "first_name" IS NOT NULL
            AND "first_name" != '' THEN 'first2'
          ELSE
            "first_name"
        END,
        "last_name" = CASE
          WHEN "id" = '1' AND "last_name" IS NOT NULL
            AND "last_name" != '' THEN 'last1'
          WHEN "id" = '2' AND "last_name" IS NOT NULL
            AND "last_name" != '' THEN 'last2'
          ELSE
            "last_name"
        END
        SQL
      end
      let(:columns)  do
        [
          { first_name: "'first1'", last_name: "'last1'", id: 1 },
          { first_name: "'first2'", last_name: "'last2'", id: 2 }
        ]
      end

      it 'generates case sql for multiple columns' do
        expect(described_class.case_sql(columns,[])).to eq sql.strip
      end

      it 'generates case sql for single column' do
        sql = <<-SQL.gsub(/\s{2,}/,' ')
        SET "first_name" = CASE
        WHEN "id" = '1' AND "first_name" IS NOT NULL
          AND "first_name" != '' THEN 'first1'
        ELSE
          "first_name"
        END,
        "last_name" = CASE
        WHEN "id" = '1' AND "last_name" IS NOT NULL
          AND "last_name" != '' THEN 'last1'
        ELSE
          "last_name"
        END
        SQL
        columns = [{ first_name: "'first1'", last_name: "'last1'", id: 1 }]
        expect(described_class.case_sql(columns,[])).to eq sql.strip
      end

      it 'generates case sql for values with quotes in them' do
        sql = <<-SQL.gsub(/\s{2,}/,' ')
        SET "first_name" = CASE
        WHEN "id" = '1' AND "first_name" IS NOT NULL
          AND "first_name" != '' THEN 'd''mello'
        ELSE
          "first_name"
        END,
        "last_name" = CASE
        WHEN "id" = '1' AND "last_name" IS NOT NULL
          AND "last_name" != '' THEN 'd''angelo'
        ELSE
          "last_name"
        END
        SQL
        columns = [{ first_name: "'d''mello'", last_name: "'d''angelo'", id: 1 }]
        expect(described_class.case_sql(columns,[])).to eq sql.strip
      end

      context 'when there is date column' do
        let(:sql) do
          <<-SQL.gsub(/\s{2,}/,' ')
        SET "first_name" = CASE
          WHEN "id" = '1' AND "first_name" IS NOT NULL
            AND "first_name" != '' THEN 'first1'
          WHEN "id" = '2' AND "first_name" IS NOT NULL
            AND "first_name" != '' THEN 'first2'
          ELSE
            "first_name"
        END,
        "date_of_birth" = CASE
          WHEN "id" = '1' AND "date_of_birth" IS NOT NULL
            THEN "date_of_birth" + INTERVAL '31 DAYS'
          WHEN "id" = '2' AND "date_of_birth" IS NOT NULL
            THEN "date_of_birth" + INTERVAL '31 DAYS'
          ELSE
            "date_of_birth"
        END
          SQL
        end
        let(:columns)  do
          [
            { first_name: "'first1'", date_of_birth: "31", id: 1 },
            { first_name: "'first2'", date_of_birth: "31", id: 2 }
          ]
        end

        it 'does not include the not empty constraint' do
          date_columns = [:date_of_birth, :date_of_death]
          expect(described_class.case_sql(columns, date_columns)).to eq sql.strip
        end
      end
    end

    describe '.update_sql' do
      sql_query = <<-SQL.gsub(/\s{2,}/,' ')
      UPDATE users
      SET first_name='fake first name',
        last_name='fake last name'
      WHERE id=1;
      SQL

      it 'returns the sql statement for updating a single row' do
        obfuscated_columns = { first_name: "'fake first name'",
                               last_name: "'fake last name'" }
        expect(described_class.update_sql(:users, 1 , obfuscated_columns)).
          to eq sql_query.strip
      end

      it 'updates date field using interval' do
        sql_query = <<-SQL.gsub(/\s{2,}/,' ')
      UPDATE users
      SET first_name='fake first name',
        last_name='fake last name',
        date_of_birth=date_of_birth + INTERVAL '31 DAYS'
      WHERE id=1;
        SQL
        obfuscated_columns = { first_name: "'fake first name'",
                               last_name: "'fake last name'",
                               date_of_birth: "31"}
        date_columns = [:date_of_birth]
        expect(described_class.update_sql(:users,
                                          1,
                                          obfuscated_columns,
                                          date_columns)).
                                         to eq sql_query.strip
      end
    end

    describe '.column_sql' do
      it 'returns the sql statement for updating the column values' do
        sql_query = <<-SQL.gsub(/\s{2,}/,' ')
        SET
        first_name='fake first name',
        last_name='fake last name'
        SQL
        obfuscated_columns = { first_name: "'fake first name'",
                               last_name: "'fake last name'" }
        date_columns = [:date_of_birth]
        expect(described_class.column_sql(obfuscated_columns, date_columns)).
          to eq sql_query.strip
      end

      it 'updates date fields using interval method' do
        sql_query = <<-SQL.gsub(/\s{2,}/,' ')
        SET
        first_name='fake first name',
        last_name='fake last name',
        date_of_birth=date_of_birth + INTERVAL '31 DAYS'
        SQL
        obfuscated_columns = { first_name: "'fake first name'",
                               last_name: "'fake last name'",
                               date_of_birth: "31"}
        date_columns = [:date_of_birth]
        expect(described_class.column_sql(obfuscated_columns, date_columns)).
          to eq sql_query.strip
      end
    end
  end
end

