require 'spec_helper'
require 'db_obfuscation/util/trigger'

def create_trigger_function
  DbObfuscation::DB.run <<-sql
    create or replace function fake_function() returns trigger
    as $$ begin end $$ language plpgsql;
  sql
end

def drop_trigger_function
  DbObfuscation::DB.run <<-sql
    drop function fake_function()
  sql
end

def create_trigger(table_name)
  create_trigger_function

  <<-sql
    CREATE TRIGGER audit_#{table_name}
    AFTER UPDATE ON #{table_name}
    FOR EACH ROW
    EXECUTE PROCEDURE fake_function();
  sql
end

def drop_trigger(table_name)
  DbObfuscation::DB.run <<-sql
   DROP TRIGGER audit_#{table_name}
   ON #{table_name}
  sql

  drop_trigger_function
end

def trigger_status
  <<-sql
   SELECT triggers.tgenabled as status,
   tables.relname as table_name
   FROM pg_trigger triggers, pg_class tables
   WHERE triggers.tgrelid = tables.oid
   AND tables.relname !~ '^pg_'
   AND triggers.tgname LIKE 'audit_%'
  sql
end

module DbObfuscation::Util
  describe Trigger do
    around(:each) do |example|
      DbObfuscation::DB.run(create_trigger('table_1')) rescue nil
      DbObfuscation::DB.run(create_trigger('table_2')) rescue nil

      example.run

      DbObfuscation::DB.run(drop_trigger('table_1')) rescue nil
      DbObfuscation::DB.run(drop_trigger('table_2')) rescue nil
    end

    it 'returns tables with triggers' do
      tables_with_triggers = ['table_1', 'table_2']
      expect(described_class.tables).to match_array(
        tables_with_triggers
      )
    end

    it 'drops all the triggers in the database' do
      described_class.drop(:all)
      expect(described_class.tables).to eq([])
    end

    it 'drops triggers for given table' do
      described_class.drop('table_1')
      expect(described_class.tables).to eq(['table_2'])
    end

    it 'disable trigger for given table' do
      status = [
        { status: 'D', table_name: 'table_1' },
        { status: 'O', table_name: 'table_2' },
      ]
      described_class.disable('table_1')
      expect(DbObfuscation::DB[trigger_status].all).to match_array(status)
    end

    it 'disables triggers for all the tables' do
      status = [
        { status: 'D', table_name: 'table_1' },
        { status: 'D', table_name: 'table_2' },
      ]
      described_class.disable(:all)
      expect(DbObfuscation::DB[trigger_status].all).to match_array(status)
    end

    it 'enables trigger for given table' do
      status = [
        { status: 'O', table_name: 'table_1' },
        { status: 'D', table_name: 'table_2' },
      ]
      described_class.disable(:all)
      described_class.enable('table_1')
      expect(DbObfuscation::DB[trigger_status].all).to match_array(status)
    end

    it 'enables all the triggers' do
      status = [
        { status: 'O', table_name: 'table_1' },
        { status: 'O', table_name: 'table_2' },
      ]
      described_class.disable(:all)
      described_class.enable(:all)
      expect(DbObfuscation::DB[trigger_status].all).to match_array(status)
    end

    describe '.trigger?' do
      it 'returns true if table has a trigger' do
        expect(described_class.exists?('table_1')).to eq true
      end

      it 'returns false if table does not have a trigger' do
        expect(described_class.exists?('whitelisted_table_1')).
          to eq false
      end
    end
  end
end
